<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\MAN\Utilities\Resposta;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\MAN\Services\TransmissaoNfceNewService;
use Teknisa\MAN\Factories\TransmissaoNFCE;

class TransmissaoNfceNewController {
    
    private $transmissaoService, $utilService, $entityManager, $notaFiscal;
    
    private $defaultFields = array(
        "SITUACAO"        => "P_R_T_A_D_C",
        "CDFILIAL"        => "T",
        "CDCAIXA"         => "T",
        "IDMODOPERACNFCE" => "T",
        "IDSITUVENDA"     => "T",
        "NRINSCRCONS"     => "T",
        "NRNOTAFISCALCE"  => "T"
    );
    
    public static $logMessages = array();
    
    public function __construct (Utilities\Util $utilService, TransmissaoNfceNewService $transmissaoService, TransmissaoNFCE $notaFiscal, $instanceManager){
        $this->transmissaoService = $transmissaoService;
        $this->utilService = $utilService;
        $this->notaFiscal = $notaFiscal;
        $this->instanceManager = $instanceManager;
    }
    
    public static function escreverLog ($data) {
        array_push(self::$logMessages, $data);
    }
    
    public static function escreverLogArray($array) {
        foreach ($array as $linha) {
            self::escreverLog($linha);
        }
    }
    
    public static function getLog () {
        return self::$logMessages;
    }
    
    public function buscar (Request\Filter $request, Response $response) {
        $parametros = $this->buildParameters($request);
        ini_set('memory_limit', '1G');
      
        if($parametros[':NRNOTAFISCALCE'] != 'T'){
            $parametros[':NRNOTAFISCALCE'] = str_pad($parametros[':NRNOTAFISCALCE'], 9, '0', STR_PAD_LEFT);
        }
        if(!isset($parametros[':ALLFIEDLS'])){
            $parametros[':ALLFIEDLS'] = 'T';
        }

        if(!isset($parametros[':TPCERTIFICADO'])){
            $parametros[':TPCERTIFICADO'] = 'T';
        }

        $result = $this->transmissaoService->executarQuery(TransmissaoNFCE::BUSCAR_NOTAS, $parametros);
        
        if($result->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
            $result = $result->getResult();
            if(!empty($result)){
                foreach($result as $key => $value){
                    try{
                        $xml = simplexml_load_string($value['DSARQXMLNFCE']);
                        if($xml){
                            $value2 = $this->transmissaoService->trataRetornoMensagemSefaz($value, $xml);
                            $result[$key]['DSOBSSTATUSNFCE'] = $value2['DSOBSSTATUSNFCE'];
                        }
                    }catch(\Exception $e){
                       $result[$key] = $value;
                    }
                }
            }
            
            $response->addDataSet(new DataSet('vnd20000_transmissaonfce', $result));
        }
      
    }
    
    public function buildParameters (Request\Filter $request, $default = true) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $params = array();
        foreach ($conditions as $filtro) {
            if (is_array($filtro['value'])) {
                // Especifico desse controller
                if ($filtro['columnName'] == "DTVENDA") {
                    if (!empty($filtro['value'][0]) && !empty($filtro['value'][1])) {
                        $params[':DATAINI'] = $filtro['value'][0];
                        $params[':DATAFIN'] = $filtro['value'][1];
                    }
                }
                if(strpos($filtro['columnName'], '|')){
                    $colunas = explode('|', $filtro['columnName']);
                    foreach($colunas as $k => $con){
                        $params[':ALLFIEDLS'] = $filtro['value'][0]['_ALL'];
                    }
                }else{
                  $params[':'.$filtro['columnName']] = implode("_", $filtro['value']);     
                }
            } else {
                $params[':'.$filtro['columnName']] = $filtro['value']; 
            }
        }
        
        if($default){
            // Especifico desse controller
            $params = $this->verificarFieldsIsset($params);
            $params = $this->verificarTiposVendas($params);
            $params = $this->verificarTipoOperacao($params);
            $params = $this->verificarDatas($params);
            $params = $this->verificarCamposDefault($params);
        }
        
        return $params;
    }
    
    public function correcaoNFCe(Request\Filter $request, Response $response){
        //projeto futuro para correcao de notas que dao problema
        $result = array();
        $parametros = $this->buildParameters($request);
        ini_set('memory_limit', '1G');
        if($parametros[':NRNOTAFISCALCE'] != 'T'){
            $parametros[':NRNOTAFISCALCE'] = str_pad($parametros[':NRNOTAFISCALCE'], 9, '0', STR_PAD_LEFT);
        }
        if(!isset($parametros[':ALLFIEDLS'])){
            $parametros[':ALLFIEDLS'] = 'T';
        }
        
        $result = $this->transmissaoService->executarQuery(TransmissaoNFCE::BUSCAR_NOTAS_CORRECAO, $parametros);
        if($result->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
            $result = $result->getResult();
            $response->addDataSet(new DataSet('vnd20000_transmissaonfce_correcao', $result));
        }
        
    }
    
    
    public function verificarTiposVendas($parametros) {
        $parametros[':VND'] = empty($parametros[':VND']) ? 'T' : $parametros[':VND'];
        if ($parametros[':VND'] == 'S') {
            $parametros[':IDSITUVENDA'] = 'O';
        } else if ($parametros[':VND'] == 'N') {
            $parametros[':IDSITUVENDA'] = 'C';
        } else if ($parametros[':VND'] == 'T') {
            $parametros[':IDSITUVENDA'] = 'T';
        } 
        return $parametros;
    }
    
    public function verificarFieldsIsset($parametros) {
        $parametros[':IDTPAMBNFCE'] = !isset($parametros[':IDTPAMBNFCE']) ? '1' : $parametros[':IDTPAMBNFCE'];
        $parametros[':IDENVIACPF'] = !isset($parametros[':IDENVIACPF']) ? 'T' : $parametros[':IDENVIACPF'];
        return $parametros;
    }
    
    public function verificarDatas($parametros) {
        if (empty($parametros[':DATAINI']) || empty($parametros[':DATAFIN'])) {
            $parametros[':DATAINI'] = 
            $parametros[':DATAFIN'] = date('d/m/Y');
        }
        return $parametros;
    }
    
    public function verificarTipoOperacao($parametros) {
        $parametros[':TIPO_EMISSAO'] = empty($parametros[':TIPO_EMISSAO']) ? 'T' : $parametros[':TIPO_EMISSAO'];
        if ($parametros[':TIPO_EMISSAO'] == 'N') {
            $parametros[':IDMODOPERACNFCE'] = '1';
        } else if ($parametros[':TIPO_EMISSAO'] == 'C') {
           $parametros[':IDMODOPERACNFCE'] = '9';
        } else if ($parametros[':TIPO_EMISSAO'] == 'T') {
            $parametros[':IDMODOPERACNFCE'] = 'T';
        } 
        return $parametros;
    }
    
    public function verificarCamposDefault($parametros) {
        $keys = array_keys($this->defaultFields);
        foreach ($keys as $key) {
            if (empty($parametros[':'.$key])) {
                $parametros[':'.$key] = $this->defaultFields[$key];
            }
        }
        return $parametros;
    }
    
    public function saveEdit(Request\DataSet $request, Response $response) {
        $parametros = array();
        $rows = $this->utilService->getParamsAsArray($request);
        $row = $rows[0];
        $NRINSCRCONS = $row['NRINSCRCONS'];
        
        if(!empty($NRINSCRCONS)){
            $NRINSCRCONS = preg_replace("/[^0-9]/", "", $NRINSCRCONS);
        	$NRINSCRCONS = str_pad($NRINSCRCONS, 11, '0', STR_PAD_LEFT);
        }
        
        if (strlen($NRINSCRCONS) < 12 && !empty($NRINSCRCONS)) {
            if (!$this->utilService->validarCPF($NRINSCRCONS)) {
                $response->addMessage(new Message("O CPF inserido não é válido.", Message::TYPE_WARNING)); 
                throw new \Exception("Erro ao validar o CPF/CNPJ");
            }
        } else if(!empty($NRINSCRCONS)){
            if (!$this->utilService->validarCNPJ($NRINSCRCONS)) {
                $response->addMessage(new Message("O CNPJ inserido não é válido.", Message::TYPE_WARNING)); 
                throw new \Exception("Erro ao validar o CPF/CNPJ");
            }  
        }
        
        $parametros[':CDFILIAL'] = $row['CDFILIAL'];
        $parametros[':CDCAIXA'] = $row['CDCAIXA'];
        $parametros[':NRSEQVENDA'] = $row['NRSEQVENDA'];
        $parametros[':NRNOTAFISCALCE'] = $row['NRNOTAFISCALCE'];
        $parametros[':NRINSCRCONS'] = !empty($NRINSCRCONS) ? $NRINSCRCONS : null;
        
        $this->transmissaoService->runQuery(true, TransmissaoNFCE::UPDATENRINSCRCONS, $parametros);
        
    }
    
    public function iniciarTentativas(Request\DataSet $request, Response $response) {
        $rows = $request->getDataSet()->getRows();
        if(!empty($rows)){
            foreach($rows as &$row) {
                if($row['IDSTATUSNFCE'] == 'P' || $row['IDSTATUSNFCE'] == 'R'){
                     $parametros = [
                        ":CDFILIAL" => $row['CDFILIAL'],
                        ":CDCAIXA" => $row['CDCAIXA'],
                        ":NRSEQVENDA" => $row['NRSEQVENDA'],
                        ":NRORG" => $row['NRORG']
                    ];
                    $this->transmissaoService->runQuery(true, TransmissaoNFCE::UPDATENRENVIOSEFAZ, $parametros);
                }
            }
            $response->addNotification(new Notification("Número de tentativa reiniciado com sucesso.", Notification::TYPE_SUCCESS));
        }else{
            $response->addNotification(new Notification("Nenhum nota teve tentativa reiniciado", Notification::TYPE_WARNING));
        }
    }
    
    public function cancelamentoSbTransmissao(Request\Row $request, Response $response){
        $retorno = new Resposta();
        $retorno->setStatus(Resposta::ERRO);
        $row = $request->getRow();
        
        $this->getLogOperacao(1, 'Cancelamento por Substituição', '', $row['CDOPERADOR']);
        
        try {
            
            if (empty($row['JUSTIFICATIVA'])) {
    			throw new \Exception("Operação Bloqueada! É necessário informar uma justificativa.");
    		}
    		if (empty($row['NRACESSONFCE2'])) {
    			throw new \Exception("Operação Bloqueada! É necessário informar a chave de substituição.");
    		}
            $result = $this->transmissaoService->cancelarNota('SB', $row);  
            
            if(!empty($result)){
                if ($result['error']) {
                    $this->getLogOperacao(2, null, 'E');
                    throw new \Exception("Ocorreu um erro ao salvar as informações de cancelamento. ".$result['message']);
                }else{
                    $this->getLogOperacao(3, null, 'S');
                    $retorno->setStatus(Resposta::SUCESSO);
                    $retorno->setMessage(
                        array(
                            'log'             => $this->getLog(),
                            'DSOBSSTATUSNFCE' => isset($result['DSOBSSTATUSNFCE']) ? $result['DSOBSSTATUSNFCE'] : ''
                        )    
                    );
                }
            }else{
                $this->getLogOperacao(2, null, 'E');
                throw new \Exception("Ocorreu um erro ao realizar cancelamento");
            }
        
        } catch (\Exception $e){
            $this->getLogOperacao(3, null, 'E', $e->getMessage());
            $retorno->setStatus(Resposta::ERRO);
            $retorno->setError(true);
            $retorno->setMessage(
                array(
                    'text' => 'Erro! <br><br>'.$e->getMessage().'<br><br>Todas as alterações foram revertidas.',
                    'log'  =>  $this->getLog()
                )
            );
        }
        $response->addDataSet(new DataSet('resultado', $retorno->toPrint()));
        
    }
    
    public function cancelarTransmissao(Request\Row $request, Response $response) {
        $retorno = new Resposta();
        $retorno->setStatus(Resposta::ERRO);
        $row = $request->getRow();
        
        $this->getLogOperacao(1, 'Cancelar Transmissão', '', $row['CDOPERADOR']);
        
        try {
            
            //sem sentido essa parametrizacao no caso de cancelamento de nota, uma vez q na tela exige uma justificativa e na query a data e sysdate
            //$cdsParaVend = $this->transmissaoService->getParamsVend($row); 
            
            $data1 = \DateTime::createFromFormat ('d/m/Y H:i:s', date('d/m/Y H:i:s'));
            $data2 = \DateTime::createFromFormat ('d/m/Y H:i:s', empty($row['DTHRPROTOCONFCE']) ? $row['DTEMISSAONFCE'] : $row['DTHRPROTOCONFCE']);
            $data2->add(new \DateInterval('P1D'));
            
            if ($data1 > $data2) {
                throw new \Exception("Operação Bloqueada! Não é permitido o cancelamento extemporâneo de NFC-e.");
            }else if (strlen($row['JUSTIFICATIVA']) < 15) {
    			throw new \Exception("Operação Bloqueada! A justificativa deve ter no mínimo 15 caracteres.");
    		}else{
                $result = $this->transmissaoService->cancelarNota('N',$row); 
                
                if(!empty($result)){
                    if ($result['error']) {
                        $this->getLogOperacao(2, null, 'E');
                        throw new \Exception("Ocorreu um erro ao salvar as informações de cancelamento. ".$result['message']);
                    }else{
                        $this->getLogOperacao(3, null, 'S');
                        $retorno->setStatus(Resposta::SUCESSO);
                        $retorno->setMessage(
                            array(
                                'log'             => $this->getLog(),
                                'DSOBSSTATUSNFCE' => isset($result['DSOBSSTATUSNFCE']) ? $result['DSOBSSTATUSNFCE'] : ''
                            )    
                        );
                    }
                }else{
                    $this->getLogOperacao(2, null, 'E');
                    throw new \Exception("Ocorreu um erro ao realizar cancelamento");
                }
            } 
            
        } catch (\Exception $e){
            $this->getLogOperacao(3, null, 'E', $e->getMessage());
            $retorno->setStatus(Resposta::ERRO);
            $retorno->setError(true);
            $retorno->setMessage(
                array(
                    'text' => 'Erro! <br><br>'.$e->getMessage().'<br><br>Todas as alterações foram revertidas.',
                    'log'  =>  $this->getLog()
                )
            );
        }
        
        $response->addDataSet(new DataSet('resultado', $retorno->toPrint()));
    }
    
    public function getLogOperacao($type, $mensage = null, $operation = null, $params = array()){
        /* $type
         1. Inicio
         2. Meio
         3. Final
        */
        
        /* $operation
         S. Sucesso
         E. Exception
        */
        
        if($type == 1){
            self::escreverLog("Log " .$mensage. " NFC-e");
            self::escreverLog("Data da operação: ".date("d/m/Y H:i:s"));
            self::escreverLog("Operador: ".$params);
            self::escreverLog("");
        }
        
        if($type == 2 && $operation == 'E' && $mensage == null){
            self::escreverLog("Operação finalizada de maneira inesperada. Data: ".date("d/m/Y H:i:s"));
        }
        
        if($type == 2 && $operation == 'E' && !empty($mensage)){
            self::escreverLog(str_pad(' ERRO! ', strlen($mensage), "-", STR_PAD_BOTH));
            self::escreverLog("| ".$mensage." |");
            self::escreverLog(str_pad('-', strlen($mensage), "-", STR_PAD_LEFT));
            self::escreverLog("Operação finalizada de maneira inesperada. Data: ".date("d/m/Y H:i:s"));
        }
        
        if($type == 3 && $operation == 'S' && $mensage == null){
            self::escreverLog("");
            self::escreverLog("## Confirmando alterações ##");
            self::escreverLog("Operação finalizada com sucesso! Data: ".date("d/m/Y H:i:s"));
        }
        
        if($type == 3 && $operation == 'S' && !empty($mensage)){
            self::escreverLog("");
            self::escreverLog("## Confirmando alterações##");
        }
        
        if($type == 3 && $operation == 'E'){
            self::escreverLog("");
            self::escreverLog(str_pad(' ERRO! ', strlen($params), "-", STR_PAD_BOTH));
            self::escreverLog("| ".$params." |");
            self::escreverLog(str_pad('-', strlen($params), "-", STR_PAD_LEFT));
            self::escreverLog("");
            self::escreverLog("## Desfazendo alterações ##");
            self::escreverLog("Operação finalizada de maneira inesperada. Data: ".date("d/m/Y H:i:s"));
        }
        
    }
    
    public function buscarNumeracao (Request\Filter $request, Response $response) {
        $parametros = $this->buildParameters($request);
        if(!isset($parametros[':NRNOTAFISCALCE'])){
            $parametros[':NRNOTAFISCALCE'] = 'T';
        }
        if(!isset($parametros[':ALLFIEDLS'])){
            $parametros[':ALLFIEDLS'] = 'T';
        }
        $result = $this->transmissaoService->executarQuery(TransmissaoNFCE::ADQNUMERACAO, $parametros);
        if($result->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
            $response->addDataSet(new DataSet('vnd20000_transmissaonfce_adqnumeracao', $result->getResult()));
        }
    }
    
    public function inutilizacaoNfe(Request\Row $request, Response $response) {
        $retorno = new Resposta();
        $retorno->setStatus(Resposta::ERRO);
        $row = $request->getRow();
        $this->getLogOperacao(1, 'Inutilização', '', $row['CDOPERADOR']);
        
        try {
            
            if(!empty($row)){
                if(!empty($row['CODNOTA'])){
                    foreach($row['CODNOTA'] as $key => $value){
                         $dadosNotaInu = explode('_', $value);
                         $parametros = [
                            ":CDFILIAL" => $dadosNotaInu[0],
                            ":DATAINI" => $row['DATAINI'],
                            ":DATAFIN" => $row['DATAFIN'],
                            ":CDCAIXA" => $dadosNotaInu[1],
                            ":NRNOTAFISCALCE" => $dadosNotaInu[2]
                        ]; 
                        
                         $result = $this->transmissaoService->executarQuery(TransmissaoNFCE::ADQNUMERACAO_FULL, $parametros);
                         if($result->error()) {
                            $this->getLogOperacao(2, $result->getMessage(), 'E');
                         } else {
                            $notaInu = $result->getResult();
                            if(!empty($notaInu)){
                                $notaInu[0]['DSOBSMOVBENECONS'] = $row['DSOBSMOVBENECONS'];
                                $result = $this->transmissaoService->inutilizarNota($notaInu[0]);
                                if(!empty($result)){
                                    if ($result['error']) {
                                        $this->getLogOperacao(2, $result['message'], 'E');
                                    }else{
                                        $this->getLogOperacao(3, "NEXT", 'S');
                                    }
                                }else{
                                    $this->getLogOperacao(2, null, 'E');
                                }
                            }
                         }
                    }
                    $this->getLogOperacao(3, null, 'S');
                }
            }
            $retorno->setStatus(Resposta::SUCESSO);
            $retorno->setMessage(
                array(
                    'log' => $this->getLog()
                )    
            );
            
        } catch (\Exception $e){
            $this->getLogOperacao(3, null, 'E', $e->getMessage());
            $retorno->setStatus(Resposta::ERRO);
            $retorno->setError(true);
            $retorno->setMessage(
                array(
                    'text' => 'Erro! <br><br>'.$e->getMessage().'<br><br>Todas as alterações foram revertidas.',
                    'log'  =>  $this->getLog()
                )
            );
        }
        
        $response->addDataSet(new DataSet('resultado', $retorno->toPrint()));
    }
    
    public function InserNfeInutilizada(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $hoje = date('d/m/Y');
        $DataInicial= date('d/m/Y', strtotime($row["DTVNDCARDS"]. ' - 30 days'));
        $row["NMRF"]=str_pad($row["NMRF"] , 9 , '0' , STR_PAD_LEFT);
        $row["NMRI"]=str_pad($row["NMRI"] , 9 , '0' , STR_PAD_LEFT);
        $row["NMRI"]=str_pad($row["SENFC"] , 3 , '0' , STR_PAD_LEFT);
        $params = array(
            "P_CDFILIAL" => $row["CDFILIAL"],
            "P_CDCAIXA" => $row["CDCAIXA"],
            "P_NRNOTAINICIAL" => $row["NMRI"],
            "P_NRNOTAFINAL" => $row["NMRF"],
            "P_CDSERIENFCE" => $row["SENFC"],
            "P_IDTPAMBNFCE" => $row["IDTPAMBNFCE"],
            "P_DTINICIAL" => $DataInicial,
            "P_DTFINAL" => $hoje,
            "P_DTENTRVENDA" => $row["DTVNDCARDS"]
        );
        $result = $this->transmissaoService->executarProcedure($params);
            
    }
    
    public function transmitirnfecf(Request\Row $request, Response $response) {
        
        $row = $request->getRow();
        $retorno = new Resposta();
        
        try{
            
            if($row['CONTROLE'] == 1){
                $this->getLogOperacao(1, 'Transmitindo', '', $row['CDOPERADORLOCAL']);
            }
            
            $result = $this->transmissaoService->transmiteNota($row);
            if($result['error']) {
                $this->getLogOperacao(2, $result['message'], 'E');
            } 
            if($row['CONTROLE'] == $row['CONTADOR']){
                $this->getLogOperacao(3, null, 'S');
            }
            
            $retorno->setStatus(Resposta::SUCESSO);
            $retorno->setMessage(
                array(
                    'log' => $this->getLog()
                )    
            );
            
        }catch(\Exception $e){
            $this->getLogOperacao(3, null, 'E', $e->getMessage());
            $retorno->setStatus(Resposta::ERRO);
            $retorno->setError(true);
            $retorno->setMessage(
                array(
                    'text' => 'Erro! <br><br>'.$e->getMessage().'<br><br>Todas as alterações foram revertidas.',
                    'log'  =>  $this->getLog()
                )
            );
        }
        
        $response->addDataSet(new DataSet('resultado', $retorno->toPrint()));
        
    }
    
    public function corrigirDuplicidade(Request\Row $request, Response $response) {
        
        $retorno = new Resposta();
        $retorno->setStatus(Resposta::ERRO);
        $row = $request->getRow();
        $this->getLogOperacao(1, 'Duplicidade', '', $row['CDOPERADOR']);
        
        try {
            
            $duplicidades = $this->transmissaoService->getDuplicidades($row);
            if($duplicidades->error()) {
                throw new \Exception("Erro! ".$result->getMessage(), 1);
            } else if(empty($duplicidades->getResult())) {
                $this->escreverLog("Não foi encontrada nenhuma nota em duplicidade.");
                $this->escreverLog("");
                $retorno->setStatus(Resposta::NAO_COMPLETADO);
                $retorno->setMessage('Operação Realizada!<br>Não foi encontrada nenhuma nota enviada em duplicidade para a SEFAZ.');
            }else{
                $this->transmissaoService->atualizaNFCE($duplicidades->getResult(), $row['rows']);
            }
            
            $this->getLogOperacao(3, null, 'S');
            $retorno->setStatus(Resposta::SUCESSO);
            $retorno->setMessage(
                array(
                    'log' => $this->getLog()
                )    
            );
        } catch (\Exception $e){
            $this->getLogOperacao(3, null, 'E', $e->getMessage());
            $retorno->setStatus(Resposta::ERRO);
            $retorno->setError(true);
            $retorno->setMessage(
                array(
                    'text' => 'Erro! <br><br>'.$e->getMessage().'<br><br>Todas as alterações foram revertidas.',
                    'log'  =>  $this->getLog()
                )
            );
        }
        $response->addDataSet(new DataSet('resultado', $retorno->toPrint()));
    }
    
    public function buscarNotasGerarChave (Request\Filter $request, Response $response) {
        $parametros = $this->buildParameters($request, false);
        if(!isset($parametros[':ALLFIEDLS'])){
            $parametros[':ALLFIEDLS'] = 'T';
        }
        
        $result = $this->transmissaoService->executarQuery(TransmissaoNFCE::ADQNUMERACAOREGCHAVE, $parametros);
        if($result->error()) {
            throw new \Exception("Erro! ".$result->getMessage(), 1);
        } else {
            $response->addDataSet(new DataSet('vnd20000_transmissaonfce_regerarchave', $result->getResult()));
        }
    }
    
     public function regerarChavesNFCe(Request\Row $request, Response $response) {
        $retorno = new Resposta();
        $retorno->setStatus(Resposta::ERRO);
        $row = $request->getRow();
        $this->getLogOperacao(1, 'Regeração de Chave de Acesso', '', $row['CDOPERADOR']);
        
        try {
            
            if(!empty($row)){
                if(!empty($row['CODNOTA'])){
                    foreach($row['CODNOTA'] as $key => $value){
                         $dadosNota = explode('_', $value);
                         $parametros = [
                            ":CDFILIAL" => $dadosNota[0],
                            ":CDCAIXA" => $dadosNota[1],
                            ":NRSEQVENDA" => $dadosNota[2]
                        ]; 
                        
                        $result = $this->transmissaoService->executarQuery(TransmissaoNFCE::ADQDADOSFILIALVENDA, $parametros);
                        if($result->error()) {
                            $this->getLogOperacao(2, $result->getMessage(), 'E');
                        } else {
                            $notaRegerarChave = $result->getResult();
                            if(!empty($notaRegerarChave)){
                                $notaRegerarChave[0]['CDOPERADOR'] = $row['CDOPERADOR'];
                                $notaRegerarChave[0]['P_NRNOTAFISCALCE'] = str_pad($row['P_NRNOTAFISCALCE'], 9, 0, STR_PAD_LEFT);
                                $notaRegerarChave[0]['P_CDSERIENFCE'] = str_pad($row['P_CDSERIENFCE'], 3, 0, STR_PAD_LEFT);
                                $notaRegerarChave[0]['NRINSJURIEST'] = preg_replace('/[^0-9]/', '',  $notaRegerarChave[0]['NRINSJURIEST']);
                                $result = $this->transmissaoService->regerarChavesNFCe($notaRegerarChave[0]);
                                $row['IDAMBTRABNFCE'] =  $notaRegerarChave[0]['IDAMBTRABNFCE'];
                                if(!empty($result)){
                                    if ($result['error']) {
                                        $this->getLogOperacao(2, $result['message'], 'E');
                                    }else{
                                        $this->getLogOperacao(3, "NEXT", 'S');
                                    }
                                }else{
                                    $this->getLogOperacao(2, null, 'E');
                                }
                                $row['P_NRNOTAFISCALCE'] += 1;
                            }
                        }
                    }
                    
                    //verifica se atualiza o NOVO CODIGO
                    if($row['P_ATUNOVOCODIGO'] == 'S'){
                        $result = $this->transmissaoService->atualizaNovoCodigo($row);
                    }
                    
                    $this->getLogOperacao(3, null, 'S');
                }
            }
            
            
            $retorno->setStatus(Resposta::SUCESSO);
            $retorno->setMessage(
                array(
                    'log' => $this->getLog()
                )    
            );
            
        } catch (\Exception $e){
            $this->getLogOperacao(3, null, 'E', $e->getMessage());
            $retorno->setStatus(Resposta::ERRO);
            $retorno->setError(true);
            $retorno->setMessage(
                array(
                    'text' => 'Erro! <br><br>'.$e->getMessage().'<br><br>Todas as alterações foram revertidas.',
                    'log'  =>  $this->getLog()
                )
            );
        }
        
        $response->addDataSet(new DataSet('resultado', $retorno->toPrint()));
    }
    
    
    
}