<?php
namespace Teknisa\MAN\Factories;

use Doctrine\DBAL\Connection;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Ger97101FechamentoCupomFactory {

    protected $connection;

    public function __construct(Connection $connection) {
        $this->connection = $connection;
    }

    /*
     * Executa procedure para aprovação dos cupons [VENDA_FINCTBFIS].
     */
    public function callAprovacaoProcedure($params) {
        $procedure = new StoredProcedure($this->connection, 'VENDA_FINCTBFIS');

        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $params["CDFILIAL"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCAIXA', Param::PARAM_INPUT, $params["CDCAIXA"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTREDUZ', Param::PARAM_INPUT, $params["DTREDUZ"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_OPERADOR', Param::PARAM_INPUT, $params["OPERADOR"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDORIGEMREDZ', Param::PARAM_INPUT, $params["IDORIGEMREDZ"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDAPROVEXTEMP', Param::PARAM_INPUT, "N", Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINIINTCONTDFS', Param::PARAM_INPUT, $params["SYSDATE"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 50));
        $procedure->addParam(new Param('P_ERROS', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 50));
        $procedure->addParam(new Param('P_TIPORECE', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 50));

        $returnParameters = $procedure->execute();

        return $returnParameters;
    }
    
    public function callAprovacaoProcedurePorLoja($params) {
        $procedure = new StoredProcedure($this->connection, 'VENDA_FINCTBFIS_LOJA');

        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $params["CDFILIAL"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDLOJA', Param::PARAM_INPUT, $params["CDLOJA"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTREDUZ', Param::PARAM_INPUT, $params["DTREDUZ"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_OPERADOR', Param::PARAM_INPUT, $params["OPERADOR"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDORIGEMREDZ', Param::PARAM_INPUT, $params["IDORIGEMREDZ"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 4000));
        $procedure->addParam(new Param('P_TIPORECE', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 4000));
        $procedure->addParam(new Param('P_ERROS', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 4000));

        $returnParameters = $procedure->execute();

        return $returnParameters;
    }


    /*
     * Executa procedure para cancelamento dos cupons [CANCVND_FINCTBFIS].
     */
    public function callCancelamentoProcedure($params) {
        $procedure = new StoredProcedure($this->connection, 'CANCVND_FINCTBFIS');

        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $params["CDFILIAL"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCAIXA', Param::PARAM_INPUT, $params["CDCAIXA"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTREDUZ', Param::PARAM_INPUT, $params["DTREDUZ"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDORIGEMREDZ', Param::PARAM_INPUT, $params["IDORIGEMREDZ"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDAPROVEXTEMP', Param::PARAM_INPUT, 'N', Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 50));

        $returnParameters = $procedure->execute();

        return $returnParameters;
    }

    /*
     * Executa procedure para cancelamento dos cupons [CANCVND_FINCTBFIS_LOJA].
     */
    public function callCancelamentoProcedurePorLoja($params) {
        $procedure = new StoredProcedure($this->connection, 'CANCVND_FINCTBFIS_LOJA');

        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $params["CDFILIAL"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDLOJA', Param::PARAM_INPUT, $params["CDLOJA"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTREDUZ', Param::PARAM_INPUT, $params["DTREDUZ"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDORIGEMREDZ', Param::PARAM_INPUT, $params["IDORIGEMREDZ"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 50));

        $returnParameters = $procedure->execute();

        return $returnParameters;
    }

    /*
     * Executa procedure para pré-validação dos cupons [PRE_VALIDACAO_REDZ].
     */
    public function callPreValidacaoProcedure($params) {
        $procedure = new StoredProcedure($this->connection, 'PRE_VALIDACAO_REDZ');

        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $params["CDFILIAL"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCAIXA', Param::PARAM_INPUT, $params["CDCAIXA"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTREDUCAOZ', Param::PARAM_INPUT, $params["DTREDUCAOZ"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDORIGEMREDZ', Param::PARAM_INPUT, $params["IDORIGEMREDZ"], Param::PARAM_TYPE_STR));
        //parametro para identificar que esta sendo chamado pelo mobile, e assim o modo de validaçao por loja é permitido
        $procedure->addParam(new Param('P_ORIGEM', Param::PARAM_INPUT, 'M', Param::PARAM_TYPE_STR));
        //parametro para identificar se a filial do cupom é válida
        $procedure->addParam(new Param('P_FILIALVALIDA', Param::PARAM_INPUT, $params["FILIALVALIDA"], Param::PARAM_TYPE_STR));

        $procedure->execute();
    }

    /*
     * Função para remover dados temporários da tabela LISTAREDZ.
     */
    public function deleteCupomListaredz($params) {
        $sql = "DELETE FROM LISTAREDZ WHERE NRLISTAREDZ = :NRLISTAREDZ AND CDLISTAREDZ = :CDLISTAREDZ AND NRORG = :NRORG";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);
    }

    /*
     * Função para retornar os dados de um cupom aprovado no sistema Fiscal.
     */
    public function getConferenciaFiscal($params) {
        $sql = "SELECT FI.CDFILIAL AS FILIAL, MV.NRORG, FI.NMFILIAL AS NMFILIAL, FI.CDFILIAL || ' - ' || FI.NMFILIAL AS DESCFILIAL, CX.CDCAIXA AS CAIXA, CX.NMCAIXA AS NMCAIXA, CX.CDCAIXA || ' - ' || CX.NMCAIXA AS DESCCAIXA, :DTREDUCAOZ AS DTREDUCAOZ, NF.NRNOTAFISC AS NUMERO, NF.DTEMISSAO AS DATA, MAX(NF.VRNOTAFISC) AS VALOR, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'00',DECODE(IP.CDFISCALNF,'2',IP.VRCONTABILNF,0),0),2)) AS VRISENTO, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'00',DECODE(IP.CDFISCALNF,'3',DECODE(IP.CDFISCOPERNF,'5405',0,IP.VRCONTABILNF),0),0),2)) AS VRNAOINCIDENCIA, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'80',IP.VRCONTABILNF,0),2)) AS VRST, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'00',IP.VRBASECALCNF,0),2)) AS VRBASEICMS, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'00',IP.VRIMPOSTONF,0),2)) AS VRICMS, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'50',IP.VRBASECALCNF,0),2)) AS VRBASEPIS, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'50',IP.VRIMPOSTONF,0),2)) AS VRPIS, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'60',IP.VRBASECALCNF,0),2)) AS VRBASECOFINS, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'60',IP.VRIMPOSTONF,0),2)) AS VRCOFINS FROM FILIAL FI, CAIXA CX, NOTAFISCAL NF, IMPOSTONF IP, IMPOSTO II,(SELECT DISTINCT (NRLANCTONFFIS) AS NRLANCTONF, (CDFILIALNFFIS) AS CDFILIALNF, NRORG FROM MOVCAIXA WHERE (CDFILIAL = :CDFILIAL) AND (CDCAIXA = :CDCAIXA) AND (DTMOVIMCAIXA = :DTREDUCAOZ) AND (NRORG = :NRORG)) MV WHERE (MV.NRLANCTONF = NF.NRLANCTONF) AND (MV.CDFILIALNF = NF.CDFILIAL) AND (MV.NRORG = NF.NRORG) AND (FI.CDFILIAL = :CDFILIAL) AND (MV.NRORG = FI.NRORG) AND (CX.CDFILIAL = :CDFILIAL) AND (CX.CDCAIXA = :CDCAIXA) AND (MV.NRORG = CX.NRORG) AND (IP.CDFILIAL = NF.CDFILIAL) AND (IP.NRLANCTONF = NF.NRLANCTONF) AND (IP.NRORG = NF.NRORG) AND (IP.CDIMPOSTO = II.CDIMPOSTO) AND (IP.NRORG = II.NRORG) GROUP BY FI.CDFILIAL, MV.NRORG, FI.NMFILIAL, FI.CDFILIAL || ' - ' || FI.NMFILIAL, CX.CDCAIXA, CX.NMCAIXA, CX.CDCAIXA || ' - ' || CX.NMCAIXA, :DTREDUCAOZ, NF.NRNOTAFISC, NF.DTEMISSAO ORDER BY DTEMISSAO";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna todos os dados obtidos.
        return $statement->fetchAll();
    }
    
    public function getConferenciaFiscalPorLoja($params) {
        $sql = "SELECT FI.CDFILIAL AS FILIAL, MV.NRORG, FI.NMFILIAL AS NMFILIAL, FI.CDFILIAL || ' - ' || FI.NMFILIAL AS DESCFILIAL, L.CDLOJA AS LOJA, L.NMLOJA AS NMLOJA, L.CDLOJA || ' - ' || L.NMLOJA AS DESCLOJA, :DTREDUCAOZ AS DTREDUCAOZ, NF.NRNOTAFISC AS NUMERO, NF.DTEMISSAO AS DATA, MAX(NF.VRNOTAFISC) AS VALOR, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'00',DECODE(IP.CDFISCALNF,'2',IP.VRCONTABILNF,0),0),2) ) AS VRISENTO, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'00',DECODE(IP.CDFISCALNF,'3',DECODE(IP.CDFISCOPERNF,'5405',0,IP.VRCONTABILNF),0),0),2) ) AS VRNAOINCIDENCIA , SUM(ROUND(DECODE(II.CDIMPOSTONFE,'80',IP.VRCONTABILNF,0),2) ) AS VRST, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'00',IP.VRBASECALCNF,0),2) ) AS VRBASEICMS, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'00',IP.VRIMPOSTONF,0),2) ) AS VRICMS, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'50',IP.VRBASECALCNF,0),2) ) AS VRBASEPIS, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'50',IP.VRIMPOSTONF,0),2) ) AS VRPIS, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'60',IP.VRBASECALCNF,0),2) ) AS VRBASECOFINS, SUM(ROUND(DECODE(II.CDIMPOSTONFE,'60',IP.VRIMPOSTONF,0),2) ) AS VRCOFINS FROM FILIAL FI, CAIXA CX, LOJA L, NOTAFISCAL NF, IMPOSTONF IP, IMPOSTO II, ( SELECT DISTINCT ( NRLANCTONFFIS ) AS NRLANCTONF, ( CDFILIALNFFIS ) AS CDFILIALNF, CDCAIXA, NRORG FROM MOVCAIXA WHERE ( CDFILIAL =:CDFILIAL ) AND ( CDCAIXA IN ( SELECT CDCAIXA FROM CAIXA WHERE CDFILIAL =:CDFILIAL AND CDLOJA =:CDLOJA ) ) AND ( DTMOVIMCAIXA =:DTREDUCAOZ ) AND ( NRORG =:NRORG ) ) MV WHERE ( MV.NRLANCTONF = NF.NRLANCTONF ) AND ( MV.CDFILIALNF = NF.CDFILIAL ) AND ( MV.NRORG = NF.NRORG ) AND ( FI.CDFILIAL =:CDFILIAL ) AND ( MV.NRORG = FI.NRORG ) AND ( CX.CDFILIAL =:CDFILIAL ) AND ( CX.CDCAIXA = MV.CDCAIXA ) AND ( L.CDFILIAL =:CDFILIAL ) AND ( L.CDLOJA = :CDLOJA) AND ( MV.NRORG = CX.NRORG ) AND ( IP.CDFILIAL = NF.CDFILIAL ) AND ( IP.NRLANCTONF = NF.NRLANCTONF ) AND ( IP.NRORG = NF.NRORG ) AND ( IP.CDIMPOSTO = II.CDIMPOSTO ) AND ( IP.NRORG = II.NRORG ) GROUP BY FI.CDFILIAL, MV.NRORG, FI.NMFILIAL, FI.CDFILIAL || ' - ' || FI.NMFILIAL, L.CDLOJA, L.NMLOJA, L.CDLOJA || ' - ' || L.NMLOJA, :DTREDUCAOZ, NF.NRNOTAFISC, NF.DTEMISSAO ORDER BY DTEMISSAO";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna todos os dados obtidos.
        return $statement->fetchAll();
    }

    /*
     * Função para retornar os dados de um cupom aprovado no sistema Contabilidade.
     */
    public function getConferenciaLancamentosContabeis($params) {
        $sql = "SELECT FI.CDFILIAL, FI.NRORG, FI.NMFILIAL, FI.CDFILIAL || ' - ' || FI.NMFILIAL AS DESCFILIAL, CX.CDCAIXA, CX.NMCAIXA, CX.CDCAIXA || ' - ' || CX.NMCAIXA AS DESCCAIXA, LL.DTLANCLOTE, LL.CDEMPRESA, CL.IDLOTE, CL.CDLOTE, LL.NRLANCDEFI, LL.DTLANCLOTE AS DTREDUCAOZ, LL.NRLANCLOTE, LL.NRSEQLANLOTE, LL.CDCONTCTBL, CT.NMCONTCTBL, CT.CDCONTCTBL || ' - ' || CT.NMCONTCTBL AS DESCCONTCTBL, ABS(LL.VRLANCLOTE) AS VRLANCLOTE, LL.DSHISTLOTE, NVL(DECODE(LL.VRLANCLOTE, ABS(LL.VRLANCLOTE), 'CR', 'DB'),'') AS TPVALOR FROM RELACNFLANCLOTE RL, LANCLOTE LL, CAPALOTE CL, FILIAL FI, CAIXA CX, CONTCTBL CT,(SELECT DISTINCT NRLANCTONF, CDFILIALNF, NRORG FROM MOVCAIXA WHERE CDFILIAL = :CDFILIAL AND CDCAIXA = :CDCAIXA AND DTMOVIMCAIXA = :DTREDUCAOZ AND NRORG = :NRORG) MV WHERE MV.NRLANCTONF = RL.NRLANCTONF AND MV.CDFILIALNF = RL.CDFILIAL AND MV.NRORG = RL.NRORG AND RL.CDEMPRESA = LL.CDEMPRESA AND RL.CDLOTE = LL.CDLOTE AND RL.NRLANCLOTE = LL.NRLANCLOTE AND RL.NRORG = LL.NRORG AND LL.CDEMPRESA = CL.CDEMPRESA AND LL.CDLOTE = CL.CDLOTE AND LL.NRORG = CL.NRORG AND LL.CDCONTCTBL = CT.CDCONTCTBL(+) AND LL.NRORG = CT.NRORG(+) AND FI.CDFILIAL = :CDFILIAL AND FI.NRORG = :NRORG AND CX.CDFILIAL = :CDFILIAL AND CX.CDCAIXA = :CDCAIXA AND CX.NRORG = :NRORG UNION ALL SELECT FI.CDFILIAL, FI.NRORG, FI.NMFILIAL, FI.CDFILIAL || ' - ' || FI.NMFILIAL AS DESCFILIAL, CX.CDCAIXA, CX.NMCAIXA, CX.CDCAIXA || ' - ' || CX.NMCAIXA AS DESCCAIXA, LL.DTLANCLOTE, LL.CDEMPRESA, CL.IDLOTE, CL.CDLOTE, LL.NRLANCDEFI, LL.DTLANCLOTE, LL.NRLANCLOTE, LL.NRSEQLANLOTE, LL.CDCONTCTBL, CT.NMCONTCTBL, CT.CDCONTCTBL || ' - ' || CT.NMCONTCTBL AS DESCCONTCTBL, ABS(LL.VRLANCLOTE) AS VRLANCLOTE, LL.DSHISTLOTE, NVL(DECODE(LL.VRLANCLOTE, ABS(LL.VRLANCLOTE), 'CR', 'DB'),'') AS TPVALOR FROM RELACNFLANCLOTE RL, LANCLOTE LL, CAPALOTE CL, FILIAL FI, CAIXA CX, CONTCTBL CT, (SELECT DISTINCT (NRLANCTONFFIS) NRLANCTONF, (CDFILIALNFFIS) CDFILIALNF, NRORG FROM MOVCAIXA WHERE CDFILIAL = :CDFILIAL AND CDCAIXA = :CDCAIXA AND DTMOVIMCAIXA = :DTREDUCAOZ AND NRORG = :NRORG) MV WHERE MV.NRLANCTONF = RL.NRLANCTONF AND MV.CDFILIALNF = RL.CDFILIAL AND MV.NRORG = RL.NRORG AND RL.CDEMPRESA = LL.CDEMPRESA AND RL.CDLOTE = LL.CDLOTE AND RL.NRLANCLOTE = LL.NRLANCLOTE AND RL.NRORG = LL.NRORG AND LL.CDEMPRESA = CL.CDEMPRESA AND LL.CDLOTE = CL.CDLOTE AND LL.NRORG = CL.NRORG AND LL.CDCONTCTBL = CT.CDCONTCTBL(+) AND LL.NRORG = CT.NRORG(+) AND FI.CDFILIAL = :CDFILIAL AND FI.NRORG = :NRORG AND CX.CDFILIAL = :CDFILIAL AND CX.CDCAIXA = :CDCAIXA AND CX.NRORG = :NRORG ORDER BY DTLANCLOTE, NRLANCLOTE, NRSEQLANLOTE";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna todos os dados obtidos.
        return $statement->fetchAll();
    }
    
    public function getConferenciaLancamentosContabeisPorLoja($params) {
        $sql = "SELECT FI.CDFILIAL, FI.NRORG, FI.NMFILIAL, FI.CDFILIAL || ' - ' || FI.NMFILIAL AS DESCFILIAL, L.CDLOJA, L.NMLOJA, L.CDLOJA || ' - ' || NMLOJA AS DESCLOJA, LL.DTLANCLOTE, LL.CDEMPRESA, CL.IDLOTE, CL.CDLOTE, LL.NRLANCDEFI, LL.DTLANCLOTE AS DTREDUCAOZ, LL.NRLANCLOTE, LL.NRSEQLANLOTE, LL.CDCONTCTBL, CT.NMCONTCTBL, CT.CDCONTCTBL || ' - ' || CT.NMCONTCTBL AS DESCCONTCTBL, ABS(LL.VRLANCLOTE) AS VRLANCLOTE, LL.DSHISTLOTE, NVL(DECODE(LL.VRLANCLOTE,ABS(LL.VRLANCLOTE),'CR','DB'),'') AS TPVALOR FROM RELACNFLANCLOTE RL, LANCLOTE LL, CAPALOTE CL, FILIAL FI, LOJA L, CONTCTBL CT, ( SELECT DISTINCT NRLANCTONF, CDFILIALNF, NRORG FROM MOVCAIXA WHERE CDFILIAL =:CDFILIAL AND CDCAIXA IN (SELECT CDCAIXA FROM CAIXA WHERE CDFILIAL = :CDFILIAL AND CDLOJA = :CDLOJA) AND DTMOVIMCAIXA =:DTREDUCAOZ AND NRORG =:NRORG ) MV WHERE MV.NRLANCTONF = RL.NRLANCTONF AND MV.CDFILIALNF = RL.CDFILIAL AND MV.NRORG = RL.NRORG AND RL.CDEMPRESA = LL.CDEMPRESA AND RL.CDLOTE = LL.CDLOTE AND RL.NRLANCLOTE = LL.NRLANCLOTE AND RL.NRORG = LL.NRORG AND LL.CDEMPRESA = CL.CDEMPRESA AND LL.CDLOTE = CL.CDLOTE AND LL.NRORG = CL.NRORG AND LL.CDCONTCTBL = CT.CDCONTCTBL (+) AND LL.NRORG = CT.NRORG (+) AND FI.CDFILIAL =:CDFILIAL AND FI.NRORG =:NRORG AND L.CDFILIAL =:CDFILIAL AND L.CDLOJA = :CDLOJA AND L.NRORG =:NRORG UNION ALL SELECT FI.CDFILIAL, FI.NRORG, FI.NMFILIAL, FI.CDFILIAL || ' - ' || FI.NMFILIAL AS DESCFILIAL, L.CDLOJA, L.NMLOJA, L.CDLOJA || ' - ' || NMLOJA AS DESCLOJA, LL.DTLANCLOTE, LL.CDEMPRESA, CL.IDLOTE, CL.CDLOTE, LL.NRLANCDEFI, LL.DTLANCLOTE, LL.NRLANCLOTE, LL.NRSEQLANLOTE, LL.CDCONTCTBL, CT.NMCONTCTBL, CT.CDCONTCTBL || ' - ' || CT.NMCONTCTBL AS DESCCONTCTBL, ABS(LL.VRLANCLOTE) AS VRLANCLOTE, LL.DSHISTLOTE, NVL(DECODE(LL.VRLANCLOTE,ABS(LL.VRLANCLOTE),'CR','DB'),'') AS TPVALOR FROM RELACNFLANCLOTE RL, LANCLOTE LL, CAPALOTE CL, FILIAL FI, LOJA L, CONTCTBL CT, ( SELECT DISTINCT ( NRLANCTONFFIS ) NRLANCTONF, ( CDFILIALNFFIS ) CDFILIALNF, NRORG FROM MOVCAIXA WHERE CDFILIAL =:CDFILIAL AND CDCAIXA IN (SELECT CDCAIXA FROM CAIXA WHERE CDFILIAL = :CDFILIAL AND CDLOJA = :CDLOJA) AND DTMOVIMCAIXA =:DTREDUCAOZ AND NRORG =:NRORG ) MV WHERE MV.NRLANCTONF = RL.NRLANCTONF AND MV.CDFILIALNF = RL.CDFILIAL AND MV.NRORG = RL.NRORG AND RL.CDEMPRESA = LL.CDEMPRESA AND RL.CDLOTE = LL.CDLOTE AND RL.NRLANCLOTE = LL.NRLANCLOTE AND RL.NRORG = LL.NRORG AND LL.CDEMPRESA = CL.CDEMPRESA AND LL.CDLOTE = CL.CDLOTE AND LL.NRORG = CL.NRORG AND LL.CDCONTCTBL = CT.CDCONTCTBL (+) AND LL.NRORG = CT.NRORG (+) AND FI.CDFILIAL =:CDFILIAL AND FI.NRORG =:NRORG AND L.CDFILIAL =:CDFILIAL AND L.CDLOJA = :CDLOJA AND L.NRORG =:NRORG ORDER BY DTLANCLOTE, NRLANCLOTE, NRSEQLANLOTE";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna todos os dados obtidos.
        return $statement->fetchAll();
    }

    /*
     * Função para retornar os dados de um cupom aprovado no sistema Financeiro.
     */
    public function getConferenciaRecebimentos($params) {
        $sql = "SELECT FI.CDFILIAL AS FILIAL, FI.NRORG, FI.NMFILIAL AS NMFILIAL, CX.CDFILIAL || ' - ' || FI.NMFILIAL AS DESCFILIAL, CX.CDCAIXA AS CAIXA, CX.NMCAIXA AS NMCAIXA, CX.CDCAIXA || ' - ' || CX.NMCAIXA AS DESCCAIXA, NF.DTEMISSAO, T.CDTIPORECE AS CDTIPORECE, T.NMTIPORECE, :DTREDUCAOZ AS DTREDUCAOZ, CR.VRORIGREC AS VALOR, CR.DTORIGVENREC AS VENCIMENTO, DECODE(CR.IDSTATREC,'B','Baixado','A','Aberto') AS STATUS, BO.DTLIQBOLREC AS DTBAIXA, CR.CDTPCTRECEB AS TIPOCONTA, TP.DSTPCTRECEB AS DESCRICAO FROM FILIAL FI, CAIXA CX, NOTAFISCAL NF, CONTARECEBER CR, BOLETRECEBER BO, TIPOCTRECEBER TP, TIPORECE T, TPRECBOLETO TR,(SELECT DISTINCT NRLANCTONF, CDFILIALNF, NRORG FROM MOVCAIXA WHERE (CDFILIAL = :CDFILIAL) AND (CDCAIXA = :CDCAIXA) AND (DTMOVIMCAIXA = :DTREDUCAOZ) AND (NRORG = :NRORG)) MV WHERE (MV.NRLANCTONF = NF.NRLANCTONF) AND (MV.CDFILIALNF = NF.CDFILIAL) AND (MV.NRORG = NF.NRORG ) AND (FI.CDFILIAL = :CDFILIAL ) AND (FI.NRORG = MV.NRORG ) AND (CX.CDFILIAL = :CDFILIAL ) AND (CX.CDCAIXA = :CDCAIXA ) AND (CX.NRORG = :NRORG ) AND (NF.CDFILIAL = CR.CDFILIAL ) AND (NF.NRLANCTONF = CR.NRLANCTONF ) AND (NF.NRORG = CR.NRORG ) AND (T.CDTPCTRECEB = CR.CDTPCTRECEB) AND (T.NRORG = CR.NRORG) AND (TP.CDTPCTRECEB = CR.CDTPCTRECEB) AND (TP.NRORG = CR.NRORG) AND (BO.CDFILIAL = CR.CDFILIBOLREC) AND (BO.NRLANBOLREC = CR.NRLANBOLREC) AND (BO.NRORG = CR.NRORG) AND (TR.CDFILIAL = CR.CDFILIBOLREC) AND (TR.NRLANBOLREC = CR.NRLANBOLREC) AND (TR.NRORG = CR.NRORG) AND (T.CDTIPORECE = TR.CDTIPORECE) ORDER BY NF.DTEMISSAO, CDTIPORECE, VENCIMENTO";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna todos os dados obtidos.
        return $statement->fetchAll();
    }
    
    public function getConferenciaRecebimentosPorLoja($params) {
        $sql = "SELECT FI.CDFILIAL AS FILIAL, FI.NRORG, FI.NMFILIAL AS NMFILIAL, FI.CDFILIAL || ' - ' || FI.NMFILIAL AS DESCFILIAL, L.CDLOJA, L.NMLOJA, L.CDLOJA || ' - ' || NMLOJA AS DESCLOJA, NF.DTEMISSAO, T.CDTIPORECE AS CDTIPORECE, T.NMTIPORECE, :DTREDUCAOZ AS DTREDUCAOZ, CR.VRORIGREC AS VALOR, CR.DTORIGVENREC AS VENCIMENTO, DECODE(CR.IDSTATREC,'B','Baixado','A','Aberto') AS STATUS, BO.DTLIQBOLREC AS DTBAIXA, CR.CDTPCTRECEB AS TIPOCONTA, TP.DSTPCTRECEB AS DESCRICAO FROM FILIAL FI, NOTAFISCAL NF, CONTARECEBER CR, BOLETRECEBER BO, TIPOCTRECEBER TP, TIPORECE T, TPRECBOLETO TR, LOJA L, ( SELECT DISTINCT NRLANCTONF, CDFILIALNF, NRORG FROM MOVCAIXA WHERE ( CDFILIAL =:CDFILIAL ) AND ( CDCAIXA IN (SELECT CDCAIXA FROM CAIXA WHERE CDFILIAL = :CDFILIAL AND CDLOJA = :CDLOJA) ) AND ( DTMOVIMCAIXA =:DTREDUCAOZ ) AND ( NRORG =:NRORG ) ) MV WHERE ( MV.NRLANCTONF = NF.NRLANCTONF ) AND ( MV.CDFILIALNF = NF.CDFILIAL ) AND ( MV.NRORG = NF.NRORG ) AND ( FI.CDFILIAL =:CDFILIAL ) AND ( FI.NRORG = MV.NRORG ) AND ( NF.CDFILIAL = CR.CDFILIAL ) AND ( NF.NRLANCTONF = CR.NRLANCTONF ) AND ( NF.NRORG = CR.NRORG ) AND ( T.CDTPCTRECEB = CR.CDTPCTRECEB ) AND ( T.NRORG = CR.NRORG ) AND ( TP.CDTPCTRECEB = CR.CDTPCTRECEB ) AND ( TP.NRORG = CR.NRORG ) AND ( BO.CDFILIAL = CR.CDFILIBOLREC ) AND ( BO.NRLANBOLREC = CR.NRLANBOLREC ) AND ( BO.NRORG = CR.NRORG ) AND ( TR.CDFILIAL = CR.CDFILIBOLREC ) AND ( TR.NRLANBOLREC = CR.NRLANBOLREC ) AND ( TR.NRORG = CR.NRORG ) AND ( T.CDTIPORECE = TR.CDTIPORECE ) AND ( L.CDLOJA = :CDLOJA) AND ( L.NRORG = MV.NRORG ) AND ( L.CDFILIAL = FI.CDFILIAL) ORDER BY NF.DTEMISSAO, CDTIPORECE, VENCIMENTO";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna todos os dados obtidos.
        return $statement->fetchAll();
    }

    /*
     * Função para retornar um registro da tabela LISTAREDZ.
     */
    public function getCupomListaReducaoZ($params) {
        $sql = "SELECT CDLISTAREDZ FROM LISTAREDZ WHERE CDLISTAREDZ = :CDLISTAREDZ AND NRORG = :NRORG";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna um dado, se existir.
        return $statement->fetch();
    }

    /*
     * Função para retornar o código da filial caso esteja entre as datas
     * parametrizadas na CONTOPECTBL.
     */
    public function getFilialBetweenParameterizedDates($params) {
        $sql = "SELECT FI.CDFILIAL FROM CONTOPECTBL CON, FILIAL FI WHERE FI.CDFILIAL = :CDFILIAL AND FI.NRORG = :NRORG AND CON.CDEMPRESA = FI.CDEMPRESA AND :DTREDUCAOZ BETWEEN CON.DTINIINTCONTDFS AND CON.DTFININTCONTDFS";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna um valor, caso a data esteja válida.
        return $statement->fetch();
    }

    /*
     * Função para retornar o próximo código disponível para inserir dados
     * temporários na tabela LISTAREDZ.
     */
    public function getNextNrlistaredz($params) {
        $sql = "SELECT NVL(MAX(NRLISTAREDZ), 0) + 1 AS NRLISTAREDZ FROM LISTAREDZ WHERE NRORG = :NRORG";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna a coluna com o valor da chave.
        return $statement->fetch()["NRLISTAREDZ"];
    }

    /*
     * Função para retornar o nome do tipo de recebimento para exibição na mensagem.
     */
    public function getTipoReceName($params) {
        $sql = "SELECT NMTIPORECE FROM TIPORECE WHERE CDTIPORECE = :CDTIPORECE AND NRORG = :NRORG";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna a coluna do nome.
        return $statement->fetch()['NMTIPORECE'];
    }

    /*
     * Função para retornar os dados gerados na procedure de pré-validação.
     */
    public function getParametroLiberacaoFilial($params) {
        $sql = "SELECT CDFILIAL, IDUTLIBERAPROV FROM PARAVEND WHERE NRORG = :NRORG";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Formata os dados, criando um array CDFILIAL => IDUTLIBERAPROV.
        $data = $statement->fetchAll();
        $data = array_reduce($data, function($currentArray, $row) {
            $currentArray[$row['CDFILIAL']] = $row['IDUTLIBERAPROV'];
            return $currentArray;
        }, array());

        // Retorna todos os dados obtidos.
        return $data;
    }

    /*
     * Função para retornar os dados gerados na procedure de pré-validação.
     */
    public function getPreValidacaoErrors($params) {
        $sql = "SELECT P.CDFILIAL, F.NMFILIAL, P.CDFILIAL || ' - ' || F.NMFILIAL AS DESCFILIAL, P.CDCAIXA, C.NMCAIXA, P.CDCAIXA || ' - ' || C.NMCAIXA AS DESCCAIXA, P.DTREDUCAOZ, P.NRSEQERRO, P.IDTPVALID, P.DSERRONFPRE, P.DSAJUDA, P.NRORG FROM PREVALIDCUPOM P JOIN FILIAL F ON P.CDFILIAL = F.CDFILIAL AND P.NRORG = F.NRORG JOIN CAIXA C ON P.CDCAIXA = C.CDCAIXA AND P.CDFILIAL = C.CDFILIAL AND P.NRORG = C.NRORG WHERE P.CDFILIAL   = :CDFILIAL AND P.CDCAIXA    = :CDCAIXA AND P.DTREDUCAOZ = :DTREDUCAOZ AND P.NRORG = :NRORG ORDER BY P.CDFILIAL, P.DTREDUCAOZ, P.CDCAIXA, P.IDTPVALID DESC, P.NRSEQERRO";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna todos os dados obtidos.
        return $statement->fetchAll();
    }
    
    public function getCaixaPorLoja($cupom) {
        $sql = "SELECT CDFILIAL, CDCAIXA, NRORG FROM CAIXA
                    WHERE
                    CDFILIAL =:CDFILIAL
                    AND CDLOJA =:CDLOJA
                    AND NRORG =:NRORG
                    ORDER BY CDCAIXA";

        $statement = $this->connection->prepare($sql);
        
        $params = array(
            "CDFILIAL" => $cupom["CDFILIAL"],
            "CDLOJA" => $cupom["CDLOJA"],
            "NRORG" => $cupom["NRORG"]
        );
        
        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna todos os dados obtidos.
        return $statement->fetchAll();
    }
    
    public function getMovLoja($cupom) {
        $sql = "SELECT * FROM MOVLOJA 
                    WHERE CDFILIAL = :CDFILIAL
                    AND CDLOJA = :CDLOJA
                    AND NRORG = :NRORG
                    AND DTMOVIMENTO = :DTREDUCAOZ
                    ORDER BY CDTIPORECE";

        $statement = $this->connection->prepare($sql);
        
        $params = array(
            "CDFILIAL" => $cupom["CDFILIAL"],
            "CDLOJA" => $cupom["CDLOJA"],
            "DTREDUCAOZ" => $cupom["DTREDUCAOZ"],
            "NRORG" => $cupom["NRORG"]
        );
        
        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna todos os dados obtidos.
        return $statement->fetchAll();
    }

    /*
     * Função para inserir dados temporários na tabela LISTAREDZ.
     */
    public function insertCupomListaredz($params) {
        $sql = "INSERT INTO LISTAREDZ(NRLISTAREDZ, CDLISTAREDZ, NRORG) VALUES(:NRLISTAREDZ, :CDLISTAREDZ, :NRORG)";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);
    }

    /*
     * Função para validar o mapa de apuração, dado um cupom.
     */
    public function mapaApuracaoIsValid($params) {
        $sql = "SELECT CDFILIAL FROM(SELECT DISTINCT RZ.CDFILIAL FROM IMPOSTOREDZ IR, IMPOSTO IM, REDUCAOZ RZ, MAPAAPURAC MP, FILIAL FI WHERE RZ.CDFILIAL = :CDFILIAL AND RZ.CDCAIXA = :CDCAIXA AND TRUNC(RZ.DTREDUCAOZ) = :DTREDUCAOZ AND RZ.NRORG = :NRORG AND RZ.CDFILIAL = IR.CDFILIAL AND RZ.CDCAIXA = IR.CDCAIXA AND TRUNC(RZ.DTREDUCAOZ) = TRUNC(IR.DTREDUCAOZ) AND RZ.NRECFREDZ = IR.NRECFREDZ AND RZ.NRORG = IR.NRORG AND IR.CDIMPOSTO = IM.CDIMPOSTO AND IR.NRORG = IM.NRORG AND IM.CDIMPOSTONFE = MP.CDIMPOSTO AND IM.NRORG = MP.NRORG AND RZ.CDFILIAL = FI.CDFILIAL AND MP.CDEMPRESA = FI.CDEMPRESA AND MP.NRORG = FI.NRORG AND ((MP.CDINSCESTA = FI.CDINSCESTA AND IM.IDORGARRECAD = 'E') OR (MP.CDINSCMUNI = FI.CDINSCMUNI AND IM.IDORGARRECAD = 'F') OR (MP.CDINSCESTA IS NULL AND MP.CDINSCMUNI IS NULL AND IM.IDORGARRECAD = 'F')) AND :DTREDUCAOZ BETWEEN MP.DTINIPERIODO AND MP.DTFIMPERIODO AND MP.IDBLOQMAPAPU = 'S' AND ((INSTR('SAT_NFCE',:IDORIGEMREDZ) = 0) OR (:IDORIGEMREDZ IS NULL)) UNION ALL SELECT DISTINCT RZ.CDFILIAL FROM REDUCAOZ RZ, MAPAAPURACSN MP, FILIAL F WHERE RZ.CDFILIAL = :CDFILIAL AND RZ.CDCAIXA = :CDCAIXA AND TRUNC(RZ.DTREDUCAOZ) = :DTREDUCAOZ AND RZ.NRORG = :NRORG AND F.CDFILIAL = RZ.CDFILIAL AND F.NRORG = RZ.NRORG AND MP.CDEMPRESA = F.CDEMPRESA AND MP.NRORG = F.NRORG AND TO_CHAR(MP.DTINIPERIODO,'MM/YYYY') = TO_CHAR(TO_DATE(:DTREDUCAOZ, 'DD/MM/YYYY'), 'MM/YYYY') AND MP.IDBLOQMAPAPU = 'S' AND ((INSTR('SAT_NFCE',:IDORIGEMREDZ) = 0) OR (:IDORIGEMREDZ IS NULL)) UNION ALL SELECT DISTINCT RZ.CDFILIAL FROM REDUCAOZ RZ, MAPAAPURAC MP, FILIAL F WHERE RZ.CDFILIAL = :CDFILIAL AND RZ.CDCAIXA = :CDCAIXA AND TRUNC(RZ.DTREDUCAOZ) = :DTREDUCAOZ AND RZ.NRORG = :NRORG AND F.CDFILIAL = RZ.CDFILIAL AND F.NRORG = RZ.NRORG AND MP.CDEMPRESA = F.CDEMPRESA AND MP.NRORG = F.NRORG AND ((MP.CDIMPOSTO = '50') OR (MP.CDIMPOSTO = '60')) AND TO_CHAR(MP.DTINIPERIODO,'MM/YYYY') = TO_CHAR(TO_DATE(:DTREDUCAOZ, 'DD/MM/YYYY'), 'MM/YYYY') AND MP.IDBLOQMAPAPU = 'S' AND ((INSTR('SAT_NFCE',:IDORIGEMREDZ) = 0) OR (:IDORIGEMREDZ IS NULL)) UNION ALL SELECT DISTINCT VE.CDFILIAL FROM VENDA VE, ITEMVENDA IT, ITVENDAIMPOS II, IMPOSTO IM, MAPAAPURAC MP, FILIAL FI, IMPOSTO I WHERE VE.CDFILIAL = :CDFILIAL AND VE.CDCAIXA = :CDCAIXA AND VE.DTENTRVENDA = :DTREDUCAOZ AND VE.NRORG = :NRORG AND VE.CDFILIAL = IT.CDFILIAL AND VE.CDCAIXA = IT.CDCAIXA AND VE.NRSEQVENDA = IT.NRSEQVENDA AND VE.NRORG = IT.NRORG AND IT.CDFILIAL = II.CDFILIAL AND IT.CDCAIXA = II.CDCAIXA AND IT.NRSEQVENDA = II.NRSEQVENDA AND IT.NRSEQUITVEND = II.NRSEQUITVEND AND IT.NRORG = II.NRORG AND II.CDIMPOSTO = IM.CDIMPOSTO AND II.NRORG = IM.NRORG AND IM.NRORG = MP.NRORG AND ((IM.CDIMPOSTONFE = MP.CDIMPOSTO) OR ((MP.CDIMPOSTO = '50' AND II.VRPERCOFINS > 0) OR (MP.CDIMPOSTO = '60' AND II.VRPERPIS > 0))) AND I.CDIMPOSTONFE = MP.CDIMPOSTO AND I.NRORG = MP.NRORG AND VE.CDFILIAL = FI.CDFILIAL AND VE.NRORG = FI.NRORG AND MP.CDEMPRESA = FI.CDEMPRESA AND ((MP.CDINSCESTA = FI.CDINSCESTA AND I.IDORGARRECAD = 'E') OR (MP.CDINSCMUNI = FI.CDINSCMUNI AND I.IDORGARRECAD = 'F') OR (MP.CDINSCESTA IS NULL AND MP.CDINSCMUNI IS NULL AND I.IDORGARRECAD = 'F')) AND :DTREDUCAOZ BETWEEN MP.DTINIPERIODO AND MP.DTFIMPERIODO AND MP.IDBLOQMAPAPU = 'S' AND VE.IDTPAMBNFCE = '1' AND VE.IDSTATUSNFCE = 'A' AND VE.IDTPEMISVEND IN ('N','S') AND :IDORIGEMREDZ = DECODE(VE.IDTPEMISVEND,'E','NFCE','SAT') UNION ALL SELECT DISTINCT VE.CDFILIAL FROM VENDA VE, MAPAAPURACSN MP, FILIAL F WHERE VE.CDFILIAL = :CDFILIAL AND VE.CDCAIXA = :CDCAIXA AND VE.DTENTRVENDA = :DTREDUCAOZ AND VE.NRORG = :NRORG AND F.CDFILIAL = VE.CDFILIAL AND F.NRORG = VE.NRORG AND MP.CDEMPRESA = F.CDEMPRESA AND MP.NRORG = F.NRORG AND TO_CHAR(MP.DTINIPERIODO,'MM/YYYY') = TO_CHAR(TO_DATE(:DTREDUCAOZ, 'DD/MM/YYYY'), 'MM/YYYY') AND MP.IDBLOQMAPAPU = 'S' AND VE.IDTPAMBNFCE = '1' AND VE.IDSTATUSNFCE = 'A' AND VE.IDTPEMISVEND IN ('N','S') AND :IDORIGEMREDZ = DECODE(VE.IDTPEMISVEND,'E','NFCE','SAT')) RZ";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Não retorna um valor, caso o mapa esteja válido.
        return $statement->fetch();
    }
    
    public function mapaApuracaoIsValidPorLoja($params) {
        $sql = "SELECT CDFILIAL FROM ( SELECT DISTINCT RZ.CDFILIAL FROM IMPOSTOREDZ IR, IMPOSTO IM, REDUCAOZ RZ, MAPAAPURAC MP, FILIAL FI WHERE RZ.CDFILIAL =:CDFILIAL AND RZ.CDCAIXA IN (SELECT CDCAIXA FROM CAIXA WHERE CDFILIAL = :CDFILIAL AND CDLOJA = :CDLOJA) AND TRUNC(RZ.DTREDUCAOZ) =:DTREDUCAOZ AND RZ.NRORG =:NRORG AND RZ.CDFILIAL = IR.CDFILIAL AND RZ.CDCAIXA = IR.CDCAIXA AND TRUNC(RZ.DTREDUCAOZ) = TRUNC(IR.DTREDUCAOZ) AND RZ.NRECFREDZ = IR.NRECFREDZ AND RZ.NRORG = IR.NRORG AND IR.CDIMPOSTO = IM.CDIMPOSTO AND IR.NRORG = IM.NRORG AND IM.CDIMPOSTONFE = MP.CDIMPOSTO AND IM.NRORG = MP.NRORG AND RZ.CDFILIAL = FI.CDFILIAL AND MP.CDEMPRESA = FI.CDEMPRESA AND MP.NRORG = FI.NRORG AND ( ( MP.CDINSCESTA = FI.CDINSCESTA AND IM.IDORGARRECAD = 'E' ) OR ( MP.CDINSCMUNI = FI.CDINSCMUNI AND IM.IDORGARRECAD = 'F' ) OR ( MP.CDINSCESTA IS NULL AND MP.CDINSCMUNI IS NULL AND IM.IDORGARRECAD = 'F' ) ) AND :DTREDUCAOZ BETWEEN MP.DTINIPERIODO AND MP.DTFIMPERIODO AND MP.IDBLOQMAPAPU = 'S' AND ( ( INSTR('SAT_NFCE',:IDORIGEMREDZ) = 0 ) OR (:IDORIGEMREDZ IS NULL ) ) UNION ALL SELECT DISTINCT RZ.CDFILIAL FROM REDUCAOZ RZ, MAPAAPURACSN MP, FILIAL F WHERE RZ.CDFILIAL =:CDFILIAL AND RZ.CDCAIXA IN (SELECT CDCAIXA FROM CAIXA WHERE CDFILIAL = :CDFILIAL AND CDLOJA = :CDLOJA) AND TRUNC(RZ.DTREDUCAOZ) =:DTREDUCAOZ AND RZ.NRORG =:NRORG AND F.CDFILIAL = RZ.CDFILIAL AND F.NRORG = RZ.NRORG AND MP.CDEMPRESA = F.CDEMPRESA AND MP.NRORG = F.NRORG AND TO_CHAR(MP.DTINIPERIODO,'MM/YYYY') = TO_CHAR(TO_DATE(:DTREDUCAOZ,'DD/MM/YYYY'),'MM/YYYY') AND MP.IDBLOQMAPAPU = 'S' AND ( ( INSTR('SAT_NFCE',:IDORIGEMREDZ) = 0 ) OR (:IDORIGEMREDZ IS NULL ) ) UNION ALL SELECT DISTINCT RZ.CDFILIAL FROM REDUCAOZ RZ, MAPAAPURAC MP, FILIAL F WHERE RZ.CDFILIAL =:CDFILIAL AND RZ.CDCAIXA IN (SELECT CDCAIXA FROM CAIXA WHERE CDFILIAL = :CDFILIAL AND CDLOJA = :CDLOJA) AND TRUNC(RZ.DTREDUCAOZ) =:DTREDUCAOZ AND RZ.NRORG =:NRORG AND F.CDFILIAL = RZ.CDFILIAL AND F.NRORG = RZ.NRORG AND MP.CDEMPRESA = F.CDEMPRESA AND MP.NRORG = F.NRORG AND ( ( MP.CDIMPOSTO = '50' ) OR ( MP.CDIMPOSTO = '60' ) ) AND TO_CHAR(MP.DTINIPERIODO,'MM/YYYY') = TO_CHAR(TO_DATE(:DTREDUCAOZ,'DD/MM/YYYY'),'MM/YYYY') AND MP.IDBLOQMAPAPU = 'S' AND ( ( INSTR('SAT_NFCE',:IDORIGEMREDZ) = 0 ) OR (:IDORIGEMREDZ IS NULL ) ) UNION ALL SELECT DISTINCT VE.CDFILIAL FROM VENDA VE, ITEMVENDA IT, ITVENDAIMPOS II, IMPOSTO IM, MAPAAPURAC MP, FILIAL FI, IMPOSTO I WHERE VE.CDFILIAL =:CDFILIAL AND VE.CDCAIXA IN (SELECT CDCAIXA FROM CAIXA WHERE CDFILIAL = :CDFILIAL AND CDLOJA = :CDLOJA) AND VE.DTENTRVENDA =:DTREDUCAOZ AND VE.NRORG =:NRORG AND VE.CDFILIAL = IT.CDFILIAL AND VE.CDCAIXA = IT.CDCAIXA AND VE.NRSEQVENDA = IT.NRSEQVENDA AND VE.NRORG = IT.NRORG AND IT.CDFILIAL = II.CDFILIAL AND IT.CDCAIXA = II.CDCAIXA AND IT.NRSEQVENDA = II.NRSEQVENDA AND IT.NRSEQUITVEND = II.NRSEQUITVEND AND IT.NRORG = II.NRORG AND II.CDIMPOSTO = IM.CDIMPOSTO AND II.NRORG = IM.NRORG AND IM.NRORG = MP.NRORG AND ( ( IM.CDIMPOSTONFE = MP.CDIMPOSTO ) OR ( ( MP.CDIMPOSTO = '50' AND II.VRPERCOFINS > 0 ) OR ( MP.CDIMPOSTO = '60' AND II.VRPERPIS > 0 ) ) ) AND I.CDIMPOSTONFE = MP.CDIMPOSTO AND I.NRORG = MP.NRORG AND VE.CDFILIAL = FI.CDFILIAL AND VE.NRORG = FI.NRORG AND MP.CDEMPRESA = FI.CDEMPRESA AND ( ( MP.CDINSCESTA = FI.CDINSCESTA AND I.IDORGARRECAD = 'E' ) OR ( MP.CDINSCMUNI = FI.CDINSCMUNI AND I.IDORGARRECAD = 'F' ) OR ( MP.CDINSCESTA IS NULL AND MP.CDINSCMUNI IS NULL AND I.IDORGARRECAD = 'F' ) ) AND :DTREDUCAOZ BETWEEN MP.DTINIPERIODO AND MP.DTFIMPERIODO AND MP.IDBLOQMAPAPU = 'S' AND VE.IDTPAMBNFCE = '1' AND VE.IDSTATUSNFCE = 'A' AND VE.IDTPEMISVEND IN ( 'N', 'S' ) AND :IDORIGEMREDZ = DECODE(VE.IDTPEMISVEND,'E','NFCE','SAT') UNION ALL SELECT DISTINCT VE.CDFILIAL FROM VENDA VE, MAPAAPURACSN MP, FILIAL F WHERE VE.CDFILIAL =:CDFILIAL AND VE.CDCAIXA IN (SELECT CDCAIXA FROM CAIXA WHERE CDFILIAL = :CDFILIAL AND CDLOJA = :CDLOJA) AND VE.DTENTRVENDA =:DTREDUCAOZ AND VE.NRORG =:NRORG AND F.CDFILIAL = VE.CDFILIAL AND F.NRORG = VE.NRORG AND MP.CDEMPRESA = F.CDEMPRESA AND MP.NRORG = F.NRORG AND TO_CHAR(MP.DTINIPERIODO,'MM/YYYY') = TO_CHAR(TO_DATE(:DTREDUCAOZ,'DD/MM/YYYY'),'MM/YYYY') AND MP.IDBLOQMAPAPU = 'S' AND VE.IDTPAMBNFCE = '1' AND VE.IDSTATUSNFCE = 'A' AND VE.IDTPEMISVEND IN ( 'N', 'S' ) AND :IDORIGEMREDZ = DECODE(VE.IDTPEMISVEND,'E','NFCE','SAT') ) RZ";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Não retorna um valor, caso o mapa esteja válido.
        return $statement->fetch();
    }

    /*
     * Função para retornar os dados gerados na procedure de pré-validação.
     */
    public function updateReducaoZ($params) {
        $sql = "UPDATE REDUCAOZ SET IDLIBERAPROV = :IDLIBERAPROV, CDOPERLIBERCUPOM = :CDOPERLIBERCUPOM WHERE CDFILIAL = :CDFILIAL AND CDCAIXA = :CDCAIXA AND DTREDUCAOZ = :DTREDUCAOZ AND NRORG = :NRORG";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);
    }

    /*
     * Função para retornar os dados gerados na procedure de pré-validação.
     */
    public function updateVenda($params) {
        $sql = "UPDATE VENDA SET IDLIBERAPROV = :IDLIBERAPROV, CDOPERLIBERCUPOM = :CDOPERLIBERCUPOM WHERE CDFILIAL = :CDFILIAL AND CDCAIXA = :CDCAIXA AND DTENTRVENDA = :DTENTRVENDA AND NRORG = :NRORG AND ((IDTPEMISVEND = 'N' AND :IDORIGEMREDZ = 'NFCE') OR (IDTPEMISVEND = 'S' AND :IDORIGEMREDZ = 'SAT')) AND IDTPAMBNFCE = '1'AND IDSTATUSNFCE = 'A'";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);
    }
}
