<?php

namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities;

class Confcaixa extends \Teknisa\MAN\Controllers\TeknisaCrud {
    
    protected $entityManager;
    
    public function __construct($entityManager,  Utilities\Util $util) {
        $this->entityManager = $entityManager;
        $this->util = $util;
    }
    
    public function get_Loja($conditions){
        try{
            $params = array(
                ':CDFILIAL' => $conditions['CDFILIAL'],
                ':NRORG'    => $conditions['NRORG']
            );
            $lojas = $this->entityManager->getConnection()->fetchAll(self::GET_LOJAS, $params);
            if($lojas){
                return $lojas[0];
            } else {
                return array( 
                    "FILILOJA"  =>  null,
                    "IDATIVO"   =>  null,
                    "NMLOJA"    =>  null,
                    "CDLOJA"    =>  null,
                    "CDFILIAL"  =>  null,
                    "NMFILIAL"  =>  null,
                    "NRORG"     =>  null,
                    "LABELLOJA" =>  null,
                  );
            }
        } catch(Exception $e){
            throw $e;
        }
    }
    
    public function verifyOpenCaixa($row){
        try{
            $params = array(
                ':CDFILIAL'    =>  $row['CDFILIAL'],
                ':CDLOJA'      =>  $row['CDLOJA'],
                ':NRORG'       =>  $row['NRORG'],
                ':DTABERCAIX'  =>  $row['DTCAIXA']
            );
            return $this->entityManager->getConnection()->fetchAll(self::VERIFY_CAIXA, $params);
        } catch(Exception $e){
            throw $e;
        }
    }
    
    
    public function verifyCaixaConferido($row){
        try{
            $params = array(
                ':CDFILIAL'     => $row['CDFILIAL'],
                ':CDTIPORECE'   => $row['CDTIPORECE'],
                ':CDLOJA'       => $row['CDLOJA'],
                ':NRORG'        => $row['NRORG'],
                ':DTCONFCAIXA'  => substr($row['DTCAIXA'], 0, 10)
            );
            $verify = $this->entityManager->getConnection()->fetchAll(self::VERIFY_MOVLOJA, $params);
            if(count($verify) > 0) {
                return true;
            } else {
                return false;
            }
        } catch(Exception $e){
            throw $e;
        }
    }
    
    public function insertConfCaixa($row){
        try{
            if(array_key_exists('VRCONFERENCIA_inline', $row)){
                if (gettype($row['VRCONFERENCIA_inline']) == 'string') {
                    $vrConfCaixa =  $this->moneyToDouble($row['VRCONFERENCIA_inline']);
                } else {
                    $vrConfCaixa = $row['VRCONFERENCIA_inline'];
                }
            } else {
                if (gettype($row['VRCONFERENCIA']) == 'string') {
                    $vrConfCaixa =  $this->moneyToDouble($row['VRCONFERENCIA']);
                } else {
                    $vrConfCaixa = $row['VRCONFERENCIA'];
                }
            }
            $params = array(
                ':NRSEQMOVLOJA'  => NovoCodigo::geraCodigo($this->entityManager->getConnection(), 'MOVLOJA'.$row['CDFILIAL'].$row['CDLOJA'], $row['NRORG'], 10),
                ':CDFILIAL'      => $row['CDFILIAL'],
                ':CDLOJA'        => $row['CDLOJA'],
                ':CDTIPORECE'    => $row['CDTIPORECE'],
                ':VRFECHAMENTO'  => floatval($row['FECHAMENTO']),
                ':VRSISTEMA'     => floatval($row['SISTEMA']),
                ':VRCONFERIDO'   => floatval($vrConfCaixa),
                ':IDCONFERIDO'   => 'S',
                ':DTMOVIMENTO'   => substr($row['DTCAIXA'], 0, 10),
                ':DTALTERACAO'   => date("d/m/Y"),
                ':NRORG'         => $row['NRORG']
            );
            return $this->entityManager->getConnection()->executeQuery(self::INSERT_CONFCAIXA, $params);
        } catch(Exception $e){
            throw $e;
        }
    }
    
    public function updateConfCaixa($row){
         try{
            if(array_key_exists('VRCONFERENCIA_inline', $row)){
                if (gettype($row['VRCONFERENCIA_inline']) == 'string') {
                    $vrConfCaixa =  $this->moneyToDouble($row['VRCONFERENCIA_inline']);
                } else {
                    $vrConfCaixa = $row['VRCONFERENCIA_inline'];
                }
            } else {
                if (gettype($row['VRCONFERENCIA']) == 'string') {
                    $vrConfCaixa =  $this->moneyToDouble($row['VRCONFERENCIA']);
                } else {
                    $vrConfCaixa = $row['VRCONFERENCIA'];
                }
            }
            
            
            $params = array(
                ':NRSEQMOVLOJA'  => $row['NRSEQMOVLOJA'],
                ':CDFILIAL'      => $row['CDFILIAL'],
                ':CDLOJA'        => $row['CDLOJA'],
                ':CDTIPORECE'    => $row['CDTIPORECE'],
                ':VRCONFERIDO'   => floatval($vrConfCaixa),
                ':DTMOVI'        => substr($row['DTCAIXA'], 0, 10),
                ':DTCONFCAIXA'   => date("d/m/Y"),
                ':NRORG'         => $row['NRORG']
            );
            //FAZER A PARTE DE INSERÇÃO DE VALOR NO CAIXA AQUI
            $this->entityManager->getConnection()->executeQuery(self::UPDATE_CONFCAIXA, $params);
        } catch(Exception $e){
            throw $e;
        }
    }
    
    public function moneyToDouble($valorAplicado) {
		$posVirgula = strpos($valorAplicado, ",");
		$posPont = strpos($valorAplicado, "." );
		
		if (!$posVirgula === false && !$posPont === false) {
			$valor = str_replace("." , "" , $valorAplicado ); // Primeiro tira os pontos
			$valorAplicado2 = str_replace(',','.',$valor);
		}else if(!$posVirgula === false && $posPont === false){
			$valorAplicado2 = str_replace(',','.',$valorAplicado);
		}else {
			$valorAplicado2 = $valorAplicado;
		}
		return $valorAplicado2;
	}
    
    
    const GET_LOJAS = "
        SELECT F.CDFILIAL || L.CDLOJA AS FILILOJA,  L.IDATIVO, L.NMLOJA, L.CDLOJA, L.CDFILIAL, F.NMFILIAL, L.NRORG,
        	L.CDLOJA ||' - '|| L.NMLOJA AS LABELLOJA 
        FROM LOJA L, FILIAL F
        WHERE L.CDFILIAL = F.CDFILIAL
         AND L.NRORG = F.NRORG
         AND L.NRORG = :NRORG
         AND L.CDFILIAL = :CDFILIAL
         AND F.IDSTATUSBLOQ = 'N'
        ORDER BY L.CDFILIAL, L.CDLOJA
    ";
    
    const VERIFY_CAIXA = "
        SELECT TU.CDFILIAL, TU.CDCAIXA, TU.DTABERCAIX, TU.NRORG, CX.NMCAIXA, TU.CDCAIXA||' - '||CX.NMCAIXA AS LABELCAIXA,
          CX.CDLOJA||' - '||LO.NMLOJA AS LABELLOJA
        FROM TURCAIXA TU
        JOIN CAIXA CX
         ON TU.CDCAIXA = CX.CDCAIXA
          AND TU.CDFILIAL = CX.CDFILIAL
          AND TU.NRORG = CX.NRORG
        JOIN LOJA LO
         ON LO.CDFILIAL = TU.CDFILIAL
          AND LO.NRORG = TU.NRORG
        WHERE TU.CDFILIAL = :CDFILIAL
          AND TU.NRORG = :NRORG
          AND CX.CDLOJA = :CDLOJA
          AND TRUNC(TU.DTABERCAIX) = :DTABERCAIX
          AND TU.DTFECHCAIX IS NULL
    ";   
   
    const VERIFY_MOVLOJA = "
       SELECT * FROM MOVLOJA
        WHERE CDFILIAL = :CDFILIAL
          AND CDTIPORECE = :CDTIPORECE
          AND NRORG = :NRORG
          AND DTMOVIMENTO = :DTCONFCAIXA
          AND CDLOJA = :CDLOJA
    ";
   
    const INSERT_CONFCAIXA = "
        INSERT INTO MOVLOJA
            (NRSEQMOVLOJA, CDFILIAL, CDLOJA, DTMOVIMENTO, CDTIPORECE, VRSANGRIA, VRVENDAS, VRCONFERIDO, IDCONFERIDO, DTALTERACAO, NRORG)
        VALUES
            (:NRSEQMOVLOJA, :CDFILIAL, :CDLOJA, TO_DATE(:DTMOVIMENTO, 'DD/MM/YYYY'), :CDTIPORECE, :VRFECHAMENTO, :VRSISTEMA, :VRCONFERIDO, :IDCONFERIDO, TO_DATE(:DTALTERACAO, 'DD/MM/YYYY'), :NRORG)
    ";
   
    const UPDATE_CONFCAIXA = "
        UPDATE MOVLOJA
        SET VRCONFERIDO  = :VRCONFERIDO,
            DTALTERACAO  = TO_DATE(:DTCONFCAIXA, 'DD/MM/YYYY')
        WHERE NRSEQMOVLOJA = :NRSEQMOVLOJA 
            AND CDFILIAL = :CDFILIAL
            AND CDTIPORECE = :CDTIPORECE
            AND CDLOJA = :CDLOJA
            AND DTMOVIMENTO = :DTMOVI
            AND NRORG = :NRORG
    ";
}