<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;

class RestricaoAlimentar{

    protected $entityManager;
    protected $params = array();
    
    public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
	}
	
	protected function setParams ($conditions){
	    $this->params = array();
	    foreach ($conditions as $key => $value) {
	        $this->params[':'.$key] = $value;
	    }
	}
	
	public function cadAutomaticoCons($conditions){
	    $this->setParams($conditions);
	    $this->updateResAliemtar(self::acmUpdateRal);
	    $cdsRetornaCons = $this->retornaDados(self::adqRetornaCons);
	    
	    $adqValRaliment = $this->retornaValRaliment();
	    $this->executarInsert($cdsRetornaCons, $adqValRaliment);
	}
	
	private function executarInsert($cdsRetornaCons, $adqValRaliment){
		try{
			$this->entityManager->getConnection()->beginTransaction();
		    foreach($cdsRetornaCons as $value){
				$codigo = $this->params[':CDFILIAL'].$value['CDCLIENTE'].$value['CDCONSUMIDOR'];
		    	if(!in_array($codigo,$adqValRaliment)){
		    		$params = array(
		    			':CDFILIAL'=>$this->params[':CDFILIAL'],
		    			':CDCLIENTE'=>$value['CDCLIENTE'],
		    			':CDCONSUMIDOR'=>$value['CDCONSUMIDOR'],
		    			':CDFAMILIA'=>$this->params[':CDFAMILIA'],
		    			':IDSALDNEGRAL'=>$this->params[':IDSALDNEGRAL'],
		    			':VRCONSDIARAL'=>$this->params[':VRCONSDIARAL'],
		    			':NRORG'=>$this->params[':NRORG']
		    		);
		    		$this->insertRestricao($params);
		    	}
		    }
		    $this->entityManager->getConnection()->commit();
		}catch(\Exception $e){
			$this->entityManager->getConnection()->rollback();
			throw $e;
		}
	}
	
	private function insertRestricao ($params){
		$this->entityManager->getConnection()->executeQuery(self::acmInsereRal,$params);
	}
	
	private function insertRestricaoProduto ($params){
		$this->entityManager->getConnection()->executeQuery(self::acmInsereProdRal,$params);
	}
	
	private function updateResAliemtar($query){
	    $this->entityManager->getConnection()->executeQuery($query,$this->params);
	}
	
	private function retornaDados ($query){
	   return $this->entityManager->getConnection()->fetchAll($query,$this->params);
	}
	
	private function retornaValRaliment (){
	   $retorno =  $this->entityManager->getConnection()->fetchAll(self::adqValRaliment,$this->params);
	   $codigos = array();
	   foreach ($retorno as $value) {
	       if(!in_array($value['CDCONSUMIDOR'],$codigos)){
	           array_push($codigos,$value['CDFILIAL'].$value['CDCLIENTE'].$value['CDCONSUMIDOR']);
	       }
	   }
	   
	   return $codigos;
	}
	
	private function retornaValProdRaliment (){
	   $retorno =  $this->entityManager->getConnection()->fetchAll(self::adqValProdRaliment,$this->params);
	   $codigos = array();
	   foreach ($retorno as $value) {
	       if(!in_array($value['CDPRODUTO'],$codigos)){
	           array_push($codigos,$value['CDPRODUTO']);
	       }
	   }
	   
	   return $codigos;
	}
	
	public function confirmarCadAutProduto($conditions){
		$this->setParams($conditions);
		$this->updateResAliemtar(self::acmUpdProdRal);
		$produtos = $this->retornaDados(self::adqRetornaProd);
		$valProdRalimt = $this->retornaValProdRaliment();
		$this->executarInsertProdutos($produtos, $valProdRalimt);
	}
	
	private function executarInsertProdutos($produtos, $valProdRalimt){
		try{
			$this->entityManager->getConnection()->beginTransaction();
		    foreach($produtos as $value){
		    	if(!in_array($value['CDPRODUTO'],$valProdRalimt)){
		    		$params = array(
		    			':CDFILIAL'=>$this->params[':CDFILIAL'],
		    			':CDCLIENTE'=>$this->params[':CDCLIENTE'],
		    			':CDCONSUMIDOR'=>$this->params[':CDCONSUMIDOR'],
		    			':CDFAMILISALD'=>$this->params[':CDFAMILIA'],
		    			':CDPRODUTO'=>$value['CDPRODUTO'],
		    			':QTCONSDIARAL'=>$this->params[':QTCONSDIARAL'],
		    			':NRORG'=>$this->params[':NRORG']
		    		);
		    		$this->insertRestricaoProduto($params);
		    	}
		    }
		    $this->entityManager->getConnection()->commit();
		}catch(\Exception $e){
			$this->entityManager->getConnection()->rollback();
			throw $e;
		}
	}
	
	const acmInsereProdRal = "
		INSERT
		INTO PRODRALIMENT
			(
			CDFILIAL,
			CDCLIENTE,
			CDCONSUMIDOR,
			CDFAMILISALD,
			CDPRODUTO,
			QTCONSDIARAL,
			NRORG
			)
			VALUES
			(
			:CDFILIAL,
			:CDCLIENTE,
			:CDCONSUMIDOR,
			:CDFAMILISALD,
			:CDPRODUTO,
			:QTCONSDIARAL,
			:NRORG
			)
	";
	
	const adqValProdRaliment = "
		SELECT CDFILIAL,
		CDCLIENTE,
		CDCONSUMIDOR,
		CDFAMILISALD,
		CDPRODUTO,
		QTCONSDIARAL
		FROM PRODRALIMENT
		WHERE CDFILIAL   = :CDFILIAL
		AND CDCLIENTE    = :CDCLIENTE
		AND CDCONSUMIDOR = :CDCONSUMIDOR
		AND CDFAMILISALD = :CDFAMILIA
		AND NRORG        = :NRORG
		AND CDPRODUTO BETWEEN (:CDPRODUTOINI) AND (:CDPRODUTOFIM)
	";
	
	const adqRetornaProd = "
		SELECT
		  FP.CDPRODUTO, PR.CDARVPROD, PR.NMPRODUTO
		FROM
		  FAMSALDOPROD FP, PRODUTO PR
		WHERE (FP.CDFILIAL = :CDFILIAL)
		  AND (FP.CDFAMILISALD = :CDFAMILIA)
		  AND (PR.CDPRODUTO = FP.CDPRODUTO)
		  AND (PR.NRORG = :NRORG)
		   AND PR.CDARVPROD BETWEEN (:CDARVPRODINI)  AND (:CDARVPRODFIM)
		  ORDER BY PR.CDARVPROD
	";
	
	const acmUpdProdRal = "
	  UPDATE PRODRALIMENT
	   SET QTCONSDIARAL = :QTCONSDIARAL
	   WHERE CDFILIAL   = :CDFILIAL
	   AND CDCLIENTE    = :CDCLIENTE
	   AND CDCONSUMIDOR = :CDCONSUMIDOR
	   AND CDFAMILISALD = :CDFAMILIA
	   AND NRORG		= :NRORG
	   AND CDPRODUTO    BETWEEN (:CDPRODUTOINI) AND (:CDPRODUTOFIM)
	";
	
	const acmUpdateRal = "
	    UPDATE RALIMENT
    SET IDSALDNEGRAL = :IDSALDNEGRAL,
        VRCONSDIARAL = :VRCONSDIARAL
    WHERE
      CDFILIAL = :CDFILIAL AND
      CDCLIENTE = :CDCLIENTE AND
      CDFAMILISALD = :CDFAMILIA AND
      NRORG = :NRORG AND
      ((INSTR(:CDCONSUMIDOR, CDCONSUMIDOR) > 0) OR (:TCDCONSUMIDOR = 'T'))
	";
	
	const adqRetornaCons = "
	    SELECT
  CDCLIENTE, CDCONSUMIDOR
    FROM
      CONSUMIDOR
    WHERE ((INSTR(:CDCCUSCLIE, CDCCUSCLIE) > 0) OR (:TCDCCUSCLIE = 'T'))
       AND ((INSTR(:CDTIPOCONS, CDTIPOCONS) > 0) OR (:TCDTIPOCONS = 'T'))
       AND ((INSTR(:CDCONSUMIDOR, CDCONSUMIDOR) > 0) OR (:TCDCONSUMIDOR = 'T'))
       AND (CDCLIENTE = :CDCLIENTE) AND NRORG = :NRORG
	";
	
	const adqValRaliment = "
	    SELECT
      R.CDFILIAL, R.CDFAMILISALD, R.CDCLIENTE, R.CDCONSUMIDOR
    FROM
      RALIMENT R, CONSUMIDOR C
    WHERE
           ((INSTR(:CDCCUSCLIE, C.CDCCUSCLIE) > 0) OR (:TCDCCUSCLIE = 'T'))
       AND ((INSTR(:CDTIPOCONS, C.CDTIPOCONS) > 0) OR (:TCDTIPOCONS = 'T'))
       AND ((INSTR(:CDCONSUMIDOR, C.CDCONSUMIDOR) > 0) OR (:TCDCONSUMIDOR = 'T'))
       AND (C.CDCLIENTE    = :CDCLIENTE)
       AND (C.CDCLIENTE    = R.CDCLIENTE)
       AND (C.CDCONSUMIDOR = R.CDCONSUMIDOR)
	";
	
	const acmInsereRal = "
		INSERT INTO RALIMENT
	  (CDFILIAL, CDCLIENTE, CDCONSUMIDOR, CDFAMILISALD, IDSALDNEGRAL, VRCONSDIARAL, NRORG)
	VALUES
	  (:CDFILIAL, :CDCLIENTE, :CDCONSUMIDOR, :CDFAMILIA, :IDSALDNEGRAL, :VRCONSDIARAL, :NRORG)
	";
}